package client;

import common.ChatMessageListener;
import common.TextMessage;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JFrame;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

/**
 * simple swing gui for the chat client
 */
public class Gui extends JFrame implements ChatMessageListener {

    private static final long serialVersionUID = 7550687331747761153L;

    private static final String LINE_SEPARATOR = System.getProperty("line.separator");

    private static final int ROW_COUNT = 20;

    private static final int COLUMN_COUNT = 60;

    private final Client chatClient;

    private final JTextArea outputArea;

    private final JTextField inputField;

    /**
     * creates layout
     *
     * @param title      title of the window
     * @param chatClient chatClient that is used for sending and receiving
     *                   messages
     */
    public Gui(String title, Client chatClient) {
        System.out.println("starting gui...");

        outputArea = new JTextArea(ROW_COUNT, COLUMN_COUNT);
        outputArea.setEditable(false);
        inputField = new JTextField();
        inputField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Gui.this.chatClient.send(inputField.getText());
                inputField.setText("");
            }
        });

        GroupLayout layout = new GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setAutoCreateGaps(true);
        layout.setAutoCreateContainerGaps(true);

        layout.setHorizontalGroup(
                layout.createSequentialGroup()
                      .addGroup(layout.createParallelGroup(Alignment.LEADING)
                                      .addComponent(outputArea)
                                      .addComponent(inputField))
        );

        layout.setVerticalGroup(
                layout.createSequentialGroup()
                      .addComponent(outputArea)
                      .addComponent(inputField)
        );

        // register listener so that we are informed whenever a new chat message
        // is received (observer pattern)
        chatClient.addMessageListener(this);
        this.chatClient = chatClient;

        setTitle(title);
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    }

    public void newChatMessage(TextMessage message) {
        String text = message.getSender() + " - ";

        /* FEATURE: Color */
        if (message.getColor() != null) {
            text += "[color=" + message.getColor() + ']';
        }

        text += message.getContent();

        /* FEATURE: Color */
        if (message.getColor() != null) {
            text += "[/color]";
        }

        if (!text.endsWith(LINE_SEPARATOR)) {
            text += LINE_SEPARATOR;
        }

        outputArea.append(text);
    }
}
